@@ -11,6 +11,9 @@ import feedparser, os, mimetypes, logging, tempfile, shutil
 class Manager:
 
     download_clients = []
+    downloads = None
+    feeds = None
+    libraries = None
 
     client_mimetypes = {
         'application/x-bittorrent': LibtorrentClient
@@ -26,13 +29,6 @@ class Manager:
         self.paused = self.application.is_paused()
 
     def get_status(self):
-        downloads = self.get_downloads()
-        queuedsize = sum([d.size for d in downloads if d.size])
-        if queuedsize:
-            queueddone = sum([d.downloaded for d in downloads])
-            progress = round((float(queueddone) / queuedsize) * 100, 2)
-        else:
-            progress = 0
 
         s = os.statvfs(self.get_work_directory())
         diskfree = s.f_bfree * s.f_bsize
@@ -42,10 +38,29 @@ class Manager:
         userdiskfree = s.f_bfree * s.f_bsize
         userdiskfreepct = (float(s.f_bfree) / s.f_blocks) * 100
 
-        active_downloads = len([d for d in downloads if d.active])
-        download_rate = sum([d.downloadrate for d in downloads])
-        upload_rate = sum([d.uploadrate for d in downloads])
-        connections = sum([d.connections for d in downloads])
+        queuedsize = 0
+        queueddone = 0
+        active_downloads = 0
+        download_rate = 0
+        upload_rate = 0
+        connections = 0
+
+        downloads = self.get_downloads()
+        for d in downloads:
+            if d.size:
+                queuedsize += d.size
+                queueddone += d.downloaded
+            if d.active:
+                active_downloads += 1
+            download_rate += d.downloadrate
+            upload_rate += d.uploadrate
+            connections += d.connections
+
+        if queuedsize:
+            progress = round((float(queueddone) / queuedsize) * 100, 2)
+        else:
+            progress = 0
+
         status = {'host': self.get_option(('downpour', 'interface'), 'localhost'),
                 'version': VERSION,
                 'downloads': len(downloads),
@@ -96,6 +111,7 @@ class Manager:
         d.downloaded = 0
 
         self.store.add(d)
+        self.get_downloads().append(d)
         self.store.commit()
         logging.debug(u'Added new download ' + d.description)
 
@@ -103,7 +119,9 @@ class Manager:
 
         return d.id
 
-    def get_downloads(self):
+    def get_downloads(self, flush=False):
+        if not self.downloads is None:
+            return self.downloads
         raise NotImplementedError('Manager must be subclassed')
 
     def get_download(self, id):
@@ -161,6 +179,8 @@ class Manager:
             Manager.download_clients.remove(dc)
         d.deleted = True
         self.store.commit()
+        # Flush delete items out of local cache
+        self.get_downloads(True)
         try:
             if remove_files:
                 workdir = self.get_work_directory(d)
@@ -233,12 +253,13 @@ class Manager:
         dfr = organizer.process_download(self.application.get_manager(d.user), d, dc)
         dfr.addCallback(self.process_download_complete, d)
         dfr.addErrback(self.process_download_failed, d)
+        return dfr
 
     def process_download_complete(self, result, d):
         d.imported = True
         d.status_message = None
         if d.feed and d.feed.auto_clean:
-            remove_download(d.id);
+            self.remove_download(d.id)
         logging.debug(u'Imported %s (%s)' % (d.id, d.description))
         self.store.commit()
 
@@ -411,34 +432,39 @@ class GlobalManager(Manager):
                 downloads.reverse()
                 for d in filter(lambda x: x.active, downloads):
                     if active > max_active:
-                        self.stop_download(d.id)
+                        sdfr = self.stop_download(d.id)
+                        sdfr.addCallback(self.update_status, d, Status.QUEUED)
+                        sdfr.addErrback(self.update_status, d, Status.QUEUED)
                         active = active - 1;
                     else:
                         break
 
-            # Reset transfer limits
-            if max_ulrate > 0:
-                client_ulrate = int(max_ulrate / active)
-                for d in filter(lambda x: x.active, downloads):
-                    dc = self.get_download_client(d.id)
-                    dc.set_upload_rate(client_ulrate)
-            if max_dlrate > 0:
-                client_dlrate = int(max_dlrate / active)
-                for d in filter(lambda x: x.active, downloads):
-                    dc = self.get_download_client(d.id)
-                    dc.set_download_rate(client_dlrate)
-            if max_conn > 0:
-                client_conn = int(max_conn / active)
-                for d in filter(lambda x: x.active, downloads):
-                    dc = self.get_download_client(d.id)
-                    dc.set_max_connections(client_conn)
+            if active > 0:
+                # Reset transfer limits
+                if max_ulrate > 0:
+                    client_ulrate = int(max_ulrate / active)
+                    for d in filter(lambda x: x.active, downloads):
+                        dc = self.get_download_client(d.id)
+                        dc.set_upload_rate(client_ulrate)
+                if max_dlrate > 0:
+                    client_dlrate = int(max_dlrate / active)
+                    for d in filter(lambda x: x.active, downloads):
+                        dc = self.get_download_client(d.id)
+                        dc.set_download_rate(client_dlrate)
+                if max_conn > 0:
+                    client_conn = int(max_conn / active)
+                    for d in filter(lambda x: x.active, downloads):
+                        dc = self.get_download_client(d.id)
+                        dc.set_max_connections(client_conn)
 
-    def get_downloads(self):
+    def get_downloads(self, flush=False):
         return list(self.store.find(models.Download,
             models.Download.deleted == False).order_by(models.Download.added))
 
     def get_feeds(self):
-        return list(self.store.find(models.Feed).order_by(models.Feed.name))
+        if self.feeds is None:
+            self.feeds = list(self.store.find(models.Feed).order_by(models.Feed.name))
+        return self.feeds
 
 class UserManager(Manager):
 
@@ -448,42 +474,44 @@ class UserManager(Manager):
 
     def add_download(self, d):
         d.user = self.user
-        Manager.add_download(self, d)
+        return Manager.add_download(self, d)
 
-    def get_downloads(self):
+    def get_downloads(self, flush=False):
         if self.user.admin:
             return list(self.store.find(models.Download,
                 models.Download.deleted == False
                 ).order_by(models.Download.added))
-        else:
-            return list(self.store.find(models.Download,
+        elif self.downloads is None or flush:
+            self.downloads = list(self.store.find(models.Download,
                 models.Download.deleted == False,
                 models.Download.user_id == self.user.id
                 ).order_by(models.Download.added))
+        return self.downloads
 
     def get_feeds(self):
         if self.user.admin:
             return list(self.store.find(models.Feed
                 ).order_by(models.Feed.name))
-        else:
+        elif self.feeds is None:
             return list(self.store.find(models.Feed,
                 models.Feed.user_id == self.user.id
                 ).order_by(models.Feed.name))
+        return self.feeds
 
     def get_libraries(self):
-        if self.user.admin:
-            return list(self.store.find(models.Library
-                ).order_by(models.Library.media_type))
-        else:
-            return list(self.store.find(models.Library,
+        if self.libraries is None:
+            self.libraries = list(self.store.find(models.Library,
                 models.Library.user_id == self.user.id
                 ).order_by(models.Library.media_type))
+        return self.libraries
 
     def get_library_directory(self):
         userdir = self.user.directory
         if not userdir:
             userdir = '%s/%s' % (self.get_user_directory(), self.user.username)
         userdir = os.path.expanduser(userdir)
+        if userdir[0] != '/':
+            userdir = '%s/%s' % (os.getcwd(), userdir)
         if not os.path.exists(userdir):
             os.mkdir(userdir)
         return userdir
